import pygame
import sys

class Game:
    def __init__(self, height: int = 1, width: int = 0, engine=pygame):  # supports only PyGame for now
        """
        Defines the game
        :param height: Optional, the height of the game window, starts at 1 to stop (0, 0) with width, but also allowing you to run the game
        :param width: Optional, the width of the game window, starts at 0
        :param engine: Optional, the game engine, only change if you want custom settings really, set to a module. If there is another game module for python, I will consider adding it, but for now it's just custom properties
        """
        self.width = width
        self.height = height
        self.engine = engine

        if self.engine == pygame:
            self.c = self.engine.time.Clock()
            self.d = self.engine.display
            self.s = self.d.set_mode((self.width, self.height))
        self.init = False

    def initEngine(self):
        """
        Initialises the current engine
        """
        if self.engine == pygame:
            self.engine.init()
            self.init = True

    def keyPressed(self, key: str):
        """
        Tests if a key is pressed
        :param key: The name of the key
        :return: If it is pressed
        """
        if self.engine == pygame and self.init:
            k = self.engine.key.get_pressed()
            k = k[self.engine.key.key_code(key.lower())]
            return k

    def runAtFPS(self, fps: int):
        """
        Ticks the game at a certain rate of frames per second
        :param fps: The frames per second to refresh on
        """
        if self.engine == pygame and self.init:
            self.d.flip()
            self.c.tick(fps)

    def getEvents(self):
        """
        Get the events of the game
        :return: The events since last tick
        """
        if self.engine == pygame and self.init:
            e = []
            for event in pygame.event.get():
                e.append(event)
            return e

    def run(self, gameFunction, fps: int, bgcol: tuple):
        """
        Runs the game with specified code. Once called, loops until quit.
        :param gameFunction: The function to be run once per tick, renders/updates the game
        :param fps: The frames per second max to run the game at
        """
        if self.engine == pygame and self.init:
            running = True
            while running:
                e = self.getEvents()
                for event in e:
                    if event.type == pygame.QUIT:
                        pygame.quit()
                
                self.s.fill(bgcol)

                gameFunction()

                self.runAtFPS(fps)


from .el_funcs import fixedrange, fixedindex
from .el_globs import globs
from .el_classes import pairs, component, game
from urllib.request import urlopen
import json

url = urlopen("https://raw.githubusercontent.com/endert1099/EndersUtils/main/versions.json")
versions = json.loads(url.read())

class EndersUtils():
    def __init__(self):
        self.fixedrange = fixedrange.fixedrange
        self.version = globs.LIBVERSION
        self.intversion = globs.LIBNUMVERSION
        self.pair = pairs.Pair
        self.fixedindex = fixedindex.fixedindex
        self.components = component.Components
        self.game = game.Game

# The main class used, although other instances can be made!
endersutils = EndersUtils()

if endersutils.intversion not in versions:
    raise ImportError(f"EndersUtils - Version {endersutils.intversion} is not up to date! Please use any of the following versions: {versions}")



class Components:
    def __init__(self) -> None:
        self.component = []

    def add_component(self, func, params):
        if callable(func):
            try:
                if type(params) == str:
                    params = (params, )
                else:
                    params = tuple(params)
                self.component.append([func, params])
            except Exception as e:
                raise e
        else:
            raise TypeError(f"Components must be a function or class method!")
    def run_component(self, idx: int):
        try:
            com = self.component
            com[idx][0](*com[idx][1])
        except IndexError:
            raise IndexError("Index is out of range!")
    def del_component(self, idx: int):
        try:
            self.component.pop(idx)
        except IndexError:
            raise IndexError("Index is out of range!")
    def list_components(self):
        com = self.component
        ret = ""
        for c in com:
            ret = ret + f"Function {c[0]}, given params {c[1]}, "
        print(ret)